#!/bin/sh
####################################################################
# Copyright Eyvaz Isaev
#
# Theoretical Physics Department,
# Moscow State Institute of Steel and Alloys 
# (Technological University)
# 
# Condensed Matter Theory Group,
# Uppsala University, Sweden
#
# Eyvaz.Isaev@fysik.uu.se, eyvaz_isaev@yahoo.com 
#
# set environment variables
#
. ./environment_variables
#
# As input parameters you have to specify (there are no default parameters):
#
# Mandatory parameters:
#
# Atoms - specify atoms involved in your calculations, you can specify Fe1, Fe1, Cr1,Cr2, etc.
#         N.B.! in the same order as you did it in your self-consistent input file.
# delta_e - frequency step (in cm{-1})
#
# "Temperature" file which contains
# T_start, T_end, T_delta
# initial T, final T, and delta T for calculation of the phonon contribution (F_vib) to 
# the free energy, the heat capacity (C_v), phonon entropy (S_vib), and phonon internal 
# energy (E_int)  
#
# T_list - list of temperatures for which the Debye temperature has to be calculated.
#          For lower T (do not specify T<10K !!!) calculations require more computer time,
#          starting from 10K is much faster 
#
# Optional parameters:
# Sysname - specify your system name you have considered, 
#           required to see your system name in output files
# SysInfo - Any information on your system (volume, pressure, etc.) will be reflected in the head part 
#           of output file 
#
# Input parameters for Quasiharonic calculations (fqha.in)
# PHDOS.out - Total phonon DOS, you might specify any partial phonon DOS name but only 
#             total phonon DOS is used for this purpose
# Sysname.QHA.out - Name of output file for QHA calculations
# T_start, T_end, T_step - starting T, the highest T used in QHA calculations, and T step
#
#
# To start Phonon DOS calculatons you have to edit matdin.init file (See below) to specify 
#          force constants matrix, and atomic masses
# You need also "ttrinp" file to show Brillouin Zone:
#          See ttrinp file for explanation how  you can manage it
# There are some ttrinp files for a number of popular (FCC, BCC, Simple Cubic, and HCP) latticies
# For these crystal structures you need no additional information and ttrinp file added 
# automatically.
##############################################################################  
# Mandatory parameters
# Specify Force Constants matrix

FC_file='CuH444.fc'

# Please do not change flfrq='frequency' line
# Edit only amass parameters

cat >matdyn.init <<EOF
&input
    asr='crystal',
    amass(1)=63.546,
    amass(2)=1.008,
    flfrc=$FC_file,
    flfrq='frequency'
&end
EOF

#
# Specify lattice type (used to create ttrinp file) and 
# atoms in the unit cell as they specified in scf.in file
# Specify the frequency step (delta_e) as well

Lattice='FCC'
atoms="Cu1 Cu2  H1  H2 "
delta_e=0.75

# Used for thermodynamic calculations 

cat >Temperature <<EOF
5 1500 5
EOF

# Used for Debye temperature calculations

T_list='10 15 20 25 30 35  40 45  50 55 60 70 80 90 100 200 300 400 500 \
        600 700 800 900 1000 1200 1400 1600 '
#
# Parameters required for QHA calculations

cat >fqha.in <<EOF
PHDOS.out
$Sysname.QHA.out
5 1500 5
EOF

# Optional parameters, any informatiion specific for a system studied 

Sysname='CuH'
SysInfo='V=17.45'

############################################################################
#############################################################################
#
# Below this line there is nothing to be changed manually 
#
#############################################################################
#
# Copy appropriate tetrahedra file to ttrinp 
#

case $Lattice in

FCC) cp ttrinp_fcc ttrinp ;;
fcc) cp ttrinp_fcc ttrinp ;;
BCC) cp ttrinp_bcc ttrinp ;;
bcc) cp ttrinp_bcc ttrinp ;;
SC)  cp ttrinp_sc ttrinp  ;;
sc)  cp ttrinp_sc ttrinp  ;;
HCP) c2a=`tail -1 $FC_file | cut -c 34-44`
     sed 's/X/'$c2a'/g' ttrinp_hcp > ttrinp;; 
hcp) c2a=`tail -1 $FC_file | cut -c 34-44`
     sed 's/X/'$c2a'/g' ttrinp_hcp > ttrinp;; 
*)   echo 'This lattice type is not implemented'
     exit;;
esac 

# Generate q-points
./tetra.x 

cp matdyn.init matdyn.init.tmp
cat  kpts_out >> matdyn.init.tmp
echo  EOF >>matdyn.init.tmp

mv  matdyn.init.tmp matdyn.in

echo ' Recalculating omega(q) from C(R)'
$BIN_DIR/matdyn.x < matdyn.in > matdyn.out

nmodes=`head -1 frequency | cut -c 13-16 `
nkpt=`head -1 frequency | cut -c 23-26 `

#
cat >phdos1.in <<EOF
$nkpt  $nmodes
$atoms
EOF

# Calculate partial phonon DOS
./Partial_phonon_DOS.x < phdos1.in 

#rm -f phdos1.in

cat >phdos.in <<EOF
$delta_e
$atoms
EOF

# Calculate total  phonon DOS and atom projected phonon DOS
./phonon_dos.x <frequency 

#rm -f phdos.in

# Remove NaN from  all phonon DOS files

cat >name <<EOF
PHDOS.out
EOF

cp PHDOS.out PHDOS.out.copy

./Ghost_DOS.x <name  >out 

mv out PHDOS.out
#rm -f name

for Atom in $atoms

do 

cat >name <<EOF
projected_DOS.$Atom
EOF

./Ghost_DOS.x <name  >out

mv out projected_DOS.$Atom

cp projected_DOS.$Atom projected.DOS

echo "# $Sysname $Atom  $SysInfo" >>Thermodynamics.$Atom

./Atom_Integrated_DOS.x >>Thermodynamics.$Atom

done

#rm -f name

# natoms - total number of atoms involved in your calculations
natoms=` echo "scale=0; $nmodes/3" | bc -l`

echo "## T (K) ##### Debye Temperature (K) " >>Theta_D

for TD in $T_list  

do

cat >debye.in <<EOF
$natoms
$TD
EOF

./Debye.x <projected.DOS >>Theta_D

done

#rm -f debey.in


./F_QHA.x <fqha.in 

#rm -f fqha.in

echo 'Phonon DOS and Quasiharmonic calculations have finished.' 
echo 'Now you can analyse these data using Gnuplot or xmgrace'
echo 'Enjoy!'

